local addon, env = ...

local ButtonMissionID = {}


local m = {__index = function(self, k) return rawset(self, k, {})[k] end}
local function table_get(t)
	return setmetatable(t, m)
end

local ButtonSuccess = table_get({})






local function hideme(b)
	if b then
		b:Hide()
		return 1
	else
		return nil
	end
end

local function HideThreats(parent, i)
	local name = parent:GetName().."Threat"
	i = i or 1
	while hideme(_G[name..i]) do
		i =i+1
	end
end


local function HideRewards(parent, i)
	local name = parent:GetName().."overmaxReward"
	i = i or 1
	while hideme(_G[name..i]) do
		i =i+1
	end
end

local function HideSuccess(parent, i)
	local name = parent:GetName().."Success"
	i = i or 1
	while hideme(_G[name..i]) do
		i =i+1
	end
end


local function NewThreat(parent, numThreats)
	local threat = CreateFrame("Frame", parent:GetName().."Threat"..numThreats, parent, "GarrisonAbilityCounterWithCheckTemplate")
	threat.Border:Hide()
	threat.TimeLeft:ClearAllPoints()
	threat.TimeLeft:SetPoint("TOPLEFT", threat, "BOTTOMLEFT", 0, -1)
	
	if numThreats == 1 then
		--threat:SetPoint("BOTTOMLEFT", button.Summary, "BOTTOMRIGHT", 10, 0)
		parent.Title:ClearAllPoints()
		parent.Title:SetPoint("TOPLEFT", parent, "TOPLEFT", 155, -11)
		threat:SetPoint("TOPLEFT", parent.Title, "BOTTOMLEFT", 0, -5)
	else
		threat:SetPoint("LEFT", parent:GetName().."Threat"..(numThreats-1), "RIGHT", 10, 0)
	end
	return threat
end


local function ShowThreats(button, missionID)
	
	
 	
	local enemies = select(8, C_Garrison.GetMissionInfo(missionID))
	local counterableThreats 	= GarrisonMission_DetermineCounterableThreats(missionID, LE_FOLLOWER_TYPE_GARRISON_7_0)	

	local numThreats = 0
	for i = 1, #enemies do
		local enemy = enemies[i];
		for mechanicID, mechanic in next, enemy.mechanics do
			numThreats = numThreats + 1
			local threat = _G[button:GetName().."Threat"..numThreats] or NewThreat(button, numThreats)
			GarrisonMissionButton_CheckTooltipThreat(threat, missionID, mechanicID, counterableThreats)
			threat.Icon:SetTexture(mechanic.ability.icon)
			threat:Show()
		end
	end
	
	HideThreats(button, numThreats+1)
end





local function GarrisonMissionButton_SetRewards(self, mission)
	local rewards = mission.overmaxRewards
	if (#rewards > 0) then
		local currencyMultipliers = nil;
		local goldMultiplier = nil;
		if (self.info.inProgress) then
			currencyMultipliers, goldMultiplier = select(8, C_Garrison.GetPartyMissionInfo(self.info.missionID));
		else
			currencyMultipliers = {};
		end

		local index = 1;
		for id, reward in pairs(rewards) do
		
			local Reward = _G[self:GetName().."overmaxReward"..id] or CreateFrame("Frame", self:GetName().."overmaxReward"..id, self, "GarrisonMissionListButtonRewardTemplate")
			
			local anchor = self.Rewards[#mission.rewards]
			Reward:SetPoint("RIGHT", anchor, "RIGHT", -(id)*(Reward:GetWidth()-16), 0);
			
			
			Reward.Quantity:Hide();
			Reward.Quantity:SetTextColor(HIGHLIGHT_FONT_COLOR:GetRGB());
			Reward.IconBorder:Hide();
			Reward.itemID = nil;
			Reward.currencyID = nil;
			Reward.tooltip = nil;
			if (reward.itemID) then
				Reward.itemID = reward.itemID;
				GarrisonMissionFrame_SetItemRewardDetails(Reward);
				if ( reward.quantity > 1 ) then
					Reward.Quantity:SetText(reward.quantity);
					Reward.Quantity:Show();
				end
			else
				Reward.Icon:SetTexture(reward.icon);
				Reward.title = reward.title
				if (reward.currencyID and reward.quantity) then
					local quantity = reward.quantity;
					if (reward.currencyID == 0) then
						if (goldMultiplier ~= nil) then
							quantity = quantity * goldMultiplier;
						end
						Reward.tooltip = GetMoneyString(quantity);
						Reward.Quantity:SetText(BreakUpLargeNumbers(floor(quantity / COPPER_PER_GOLD)));
						Reward.Quantity:Show();
					else
						if (currencyMultipliers[reward.currencyID] ~= nil) then
							quantity = quantity * currencyMultipliers[reward.currencyID];
						end
						Reward.currencyID = reward.currencyID;
						Reward.Quantity:SetText(quantity);
						local currencyColor = GetColorForCurrencyReward(reward.currencyID, quantity);
						Reward.Quantity:SetTextColor(currencyColor:GetRGB());
						Reward.Quantity:Show();
					end
				else
					Reward.tooltip = reward.tooltip;
					if ( reward.followerXP ) then
						Reward.Quantity:SetText(BreakUpLargeNumbers(reward.followerXP));
						Reward.Quantity:Show();
					end
				end
			end
			Reward:Show();
			index = index + 1;
		end
	end
	HideRewards(self, #rewards + 1)
end


local function AssignFollower(followerID, v, i)
	if followerID == v.followerID then 
		OrderHallMissionFrame:AssignFollowerToMission(OrderHallMissionFrame.MissionTab.MissionPage["Follower"..i], v)
		return 1
	else
		return nil
	end
end

local function ShowTooltip(self)
	local info=self.info
	--self:GetParent():LockHighlight()
	if info then
		GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 0)

		local text = info.text
		if text == "" then
			text = "% with lesser followers"
		elseif text == "C" then
			text = "Cost"
		elseif text == "T" then
			text = "Time"	
		elseif text == "XP" then
			text = "XP with lesser cost"
		elseif text == "CT" then
			text = "Cost and Time"
		elseif text == "CXP" then
			text = "Cost and XP"	
		elseif text == "TXP" then
			text = "Time and XP"
		elseif text == "CTXP" then
			text = "Cost, Time and XP"		
		else
			text = ""
		end
		GameTooltip:AddLine("Best "..text, 1, 1, 1)
		GameTooltip:AddDoubleLine("Success",  info.successChance.." %") 
		GameTooltip:AddDoubleLine("Cost",  info.cost)
		GameTooltip:AddDoubleLine("Time",  GarrisonMission_GetDurationStringCompact(info.totalTimeSeconds)) 
		GameTooltip:AddDoubleLine("XP",  info.n_xp.." * "..info.xp)
		GameTooltip:Show()
	end
end

local function HideTooltip(self)
	GameTooltip:Hide()
	--self:GetParent():UnlockHighlight()
end
 
local function AddFollowers(self)
	local info = self.info  --env.mission_success[missionID][index]
	local missionPage = OrderHallMissionFrame.MissionTab.MissionPage
	local missionInfo = missionPage.missionInfo
	local numFollowers = missionInfo.numFollowers
	
	
	
		 
 	for i=1, info.n do
		local followerID = info[i]

		missionPage:AddFollower(followerID)
  	end
end
 





 local function NewButtonSuccess(parent, i)
	local button = CreateFrame("Button", parent:GetName().."Success"..i, parent , "SecureActionButtonTemplate, UIPanelButtonTemplate")
	button:SetHeight(20)
 
 	button:SetScript("OnEnter", ShowTooltip)
	button:SetScript("OnLeave", HideTooltip)
	
	button:SetAttribute("type", "macro")
	button:SetAttribute("macrotext", "/click "..parent:GetName())
	button:SetAttribute("index", i)
	
	
	button:HookScript("OnClick", AddFollowers)
	
	

 	if i == 1 then
		button:SetPoint("LEFT", parent, "LEFT", 250, -15) 
	else
		button:SetPoint("LEFT",  parent:GetName().."Success"..(i-1), "RIGHT", 0, 0)
	end
	return button
end


local symbols ={"", "C", "T", "XP"}

local function compare_combi(n, t1, t2)
	if n ~= t2.n then
		return nil
	end
	for i=1, n do
 		if t1[i] ~= t2[i] then
			return nil
		end
	end
	return 1
end

local function ShowSuccess(parent, missionID)
	local infos = env.mission_success[missionID]
	local count = 0
 	for i=1, 4 do
		local info = infos[i]
		if info then
			count=count+1
			local button = _G[parent:GetName().."Success"..count] or NewButtonSuccess(parent, count)		
			local text = info.text
			if not text then
 				text = symbols[i]
				local n = info.n
				for j=i+1, 4 do
					local t = infos[j]
					if t and compare_combi(n, info, t) then   -- env.compare_table(info.combi, infos[j].combi) then
						wipe(t)
						infos[j] = nil
						text = text..symbols[j]
					end
				end
				info.text = text
					
			end
			
  
			
			text = info.successChance.." % "..text
			button.info = info
 			button:SetText(text)
			button:SetWidth(#text*9)
			button:Show()
		end
	end
	
	HideSuccess(parent, count+1)
end

local wt = {}
local inupdate = nil
 env.update_MissionList = function(frame, ...)
	
 	if not frame:IsVisible() or frame.CompleteDialog:IsShown() or OrderHallMissionFrame.MissionComplete:IsShown() then return
	end
 	if inupdate then return end
	inupdate = 1
	
	if not frame.showInProgress then

		env.update_pool_followers()
	end
	
   	for k, button in next, frame.listScroll.buttons do
  		local mission = button.info
		if mission then 
			--print(mission.missionID)

			if not mission.inProgress then
				local infos = env.mission_success[mission.missionID] or nil
				if not infos then
					--print("update "..k.." "..mission.missionID)
					env.find_followers(wt, mission)
				end
				if not infos or  ButtonMissionID[k] ~= mission.missionID then
				
					ShowThreats(button, mission.missionID)
					ShowSuccess(button, mission.missionID)
					
				end
			end
			
			
			if ButtonMissionID[k] ~= mission.missionID then
			
				if mission.inProgress then
					HideThreats(button)
					HideSuccess(button)
				end
				GarrisonMissionButton_SetRewards(button, mission)
				ButtonMissionID[k] = mission.missionID
			end
			

		end
	end
	
	if wt[0] then
		for i=1, wt[0] do
			local frame = wt[i]
			if frame then frame:RegisterEvent("GARRISON_FOLLOWER_LIST_UPDATE")
			end
			wt[i] = nil
		end
		wt[0] = nil
	end
	

	inupdate = nil
end
 
 
 
  






 


 
 

 hooksecurefunc(OrderHallMissionFrame.MissionTab.MissionList, "Update", env.update_MissionList)






